
/* Chrysalide - Outil d'analyse de fichiers binaires
 * raw.c - équivalent Python du fichier "arch/raw.h"
 *
 * Copyright (C) 2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "raw.h"


#include <pygobject.h>


#include <arch/raw.h>


#include "instruction.h"
#include "../helpers.h"



/* Indique si le contenu de l'instruction est du bourrage. */
static PyObject *py_arch_instruction_is_padding(PyObject *, void *);

/* Marque l'instruction comme ne contenant que du bourrage. */
static int py_arch_instruction_mark_as_padding(PyObject *, PyObject *, void *);

/* Indique si le contenu de l'instruction est un texte. */
static PyObject *py_arch_instruction_is_string(PyObject *, void *);

/* Marque l'instruction comme contenant une chaîne de texte. */
static int py_arch_instruction_mark_as_string(PyObject *, PyObject *, void *);



/******************************************************************************
*                                                                             *
*  Paramètres  : self    = classe représentant une instruction.               *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Indique si le contenu de l'instruction est du bourrage.      *
*                                                                             *
*  Retour      : Valeur associée à la propriété consultée.                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_arch_instruction_is_padding(PyObject *self, void *closure)
{
    PyObject *result;                       /* Conversion à retourner      */
    GRawInstruction *instr;                 /* Version native              */
    bool state;                             /* Etat courant à consulter    */

    instr = G_RAW_INSTRUCTION(pygobject_get(self));

    state = g_raw_instruction_is_padding(instr);

    result = state ? Py_True : Py_False;
    Py_INCREF(result);


    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                value   = valeur fournie à intégrer ou prendre en compte.    *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Marque l'instruction comme ne contenant que du bourrage.     *
*                                                                             *
*  Retour      : Bilan de l'opération pour Python.                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int py_arch_instruction_mark_as_padding(PyObject *self, PyObject *value, void *closure)
{
    bool state;                             /* Nouvel état à définir       */
    GRawInstruction *instr;                 /* Version native              */

    if (value != Py_True && value != Py_False)
        return -1;

    state = (value == Py_True);

    instr = G_RAW_INSTRUCTION(pygobject_get(self));

    g_raw_instruction_mark_as_padding(instr, state);

    return 0;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = classe représentant une instruction.               *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Indique si le contenu de l'instruction est un texte.         *
*                                                                             *
*  Retour      : Valeur associée à la propriété consultée.                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_arch_instruction_is_string(PyObject *self, void *closure)
{
    PyObject *result;                       /* Conversion à retourner      */
    GRawInstruction *instr;                 /* Version native              */
    bool state;                             /* Etat courant à consulter    */

    instr = G_RAW_INSTRUCTION(pygobject_get(self));

    state = g_raw_instruction_is_string(instr);

    result = state ? Py_True : Py_False;
    Py_INCREF(result);


    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                value   = valeur fournie à intégrer ou prendre en compte.    *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Marque l'instruction comme contenant une chaîne de texte.    *
*                                                                             *
*  Retour      : Bilan de l'opération pour Python.                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static int py_arch_instruction_mark_as_string(PyObject *self, PyObject *value, void *closure)
{
    bool state;                             /* Nouvel état à définir       */
    GRawInstruction *instr;                 /* Version native              */

    if (value != Py_True && value != Py_False)
        return -1;

    state = (value == Py_True);

    instr = G_RAW_INSTRUCTION(pygobject_get(self));

    g_raw_instruction_mark_as_string(instr, state);

    return 0;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_raw_instruction_type(void)
{
    static PyMethodDef py_raw_instruction_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_raw_instruction_getseters[] = {
        {
            "is_padding", py_arch_instruction_is_padding, py_arch_instruction_mark_as_padding,
            "Report if the instruction is seen as padding.", NULL
        },
        {
            "is_string", py_arch_instruction_is_string, py_arch_instruction_mark_as_string,
            "Report if the instruction is seen as a string.", NULL
        },
        { NULL }
    };

    static PyTypeObject py_raw_instruction_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.arch.RawInstruction",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide raw instruction for a all architectures.",

        .tp_methods     = py_raw_instruction_methods,
        .tp_getset      = py_raw_instruction_getseters,

    };

    return &py_raw_instruction_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.arch.ArchInstruction'. *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_raw_instruction(PyObject *module)
{
    PyTypeObject *py_raw_instruction_type;  /* Type Python 'RawInstruction'*/
    PyObject *dict;                         /* Dictionnaire du module      */
    PyTypeObject *base;                     /* Base parente pour héritage  */

    py_raw_instruction_type = get_python_raw_instruction_type();

    dict = PyModule_GetDict(module);

    base = get_python_arch_instruction_type();

    if (!register_class_for_pygobject(dict, G_TYPE_RAW_INSTRUCTION, py_raw_instruction_type, base))
        return false;

    return true;

}
